@@ -1,4 +1,5 @@
 import subprocess
+from typing import List, Dict
 
 class Commands():
     """ Class that specifies all PIP commands that are implemented
@@ -25,35 +26,11 @@ class Commands():
     def _uninstall(cls, name:str):
         return cls._create_cmd('pip', 'uninstall', name, '-y')
 
-    @classmethod
-    def _bulk_install(cls, name:list):
-        """ TO DO """
-        pass
-
-    @classmethod
-    def _bulk_uninstall(cls, name:list):
-        """ TO DO """
-        pass
-
-    @classmethod
-    def _install_from_file(cls, file:str):
-        """ TO DO """
-        pass
-
-    @classmethod
-    def _uninstall_from_file(cls, file:str):
-        """ TO DO """
-        pass
-
-    @classmethod
-    def _export(cls, file:str):
-        """ TO DO """
-        pass
 
 class PIP():
     """ Python wrapper for PIP installer """
 
-    __version__ = '0.24'
+    __version__ = '0.30'
     __author__ = 'Jordan Raychev'
     __email__ = 'jpraychev at gmail dot com'
     __license__ = 'MIT. Please refer to LICENSE file'
@@ -70,8 +47,10 @@ class PIP():
         return 'A Python wrapper of the pip installer'
 
     @staticmethod
-    def list_packages(pretty=False):
-        """ List all installed packages nicely formatted """
+    def list_packages(pretty=False) -> Dict:
+        """ List all installed packages. If pretty argument is set to True, 
+        formats the output with indent. """
+
         packages = dict()
         lp = Commands._list_packages()
         inst_packages = lp.stdout.split('\n')[:-1]
@@ -94,7 +73,7 @@ class PIP():
         try:
             package = Commands._install(name=package_name)
         except subprocess.CalledProcessError:
-            return f'{package_name} package does not exist\n'
+            return f'{package_name} could not be installed\n'
         return package.stdout
 
     @staticmethod
@@ -110,26 +89,47 @@ class PIP():
         return package.stdout if package.stdout else package.stderr
 
     @staticmethod
-    def bulk_install(package_names:list):
-        """ TO DO """
-        raise NotImplementedError
+    def bulk_install(package_names:List[str]) -> str:
+        """ Bulk install of packages from a list """
+
+        success = []
+        failure = []
+
+        if not isinstance(package_names, list):
+            raise TypeError(f'Expected {package_names} to be a list of strings')
+        for package_name in package_names:
+            try:
+                package = Commands._install(name=package_name)
+            except subprocess.CalledProcessError as e:
+                failure.append(package_name)
+                break
+            success.append(package_name)
+        return f'Installed packages: {success}\nNot installed packages: {failure}'
+
 
     @staticmethod
-    def bulk_uninstall(package_names):
-        """ TO DO """
-        raise NotImplementedError
+    def bulk_uninstall(package_names:List[str]) -> str:
+        """ Bulk uninstall of packages from a list """
+        success = []
+        failure = []
+        if not isinstance(package_names, list):
+            raise TypeError(f'Expected {package_names} to be a list of strings')
+        for package_name in package_names:
+            try:
+                package = Commands._uninstall(name=package_name)
+            except Exception:
+                failure.append(package_name)
+                break
+            success.append(package_name)
+        return f'Uninstalled packages: {success}\nNot uninstalled packages: {failure}'
 
     @staticmethod
-    def install_from_file(file):
-        """ TO DO """
-        raise NotImplementedError
+    def export() -> None:
+        """ Exports installed packages to requirements.txt file """
 
-    @staticmethod
-    def uninstall_from_file(file):
-        """ TO DO """
-        raise NotImplementedError
+        lp = Commands._list_packages()
+        installed_packages = lp.stdout.split('\n')[:-1]
 
-    @staticmethod
-    def export(file):
-        """ TO DO """
-        raise NotImplementedError
\ No newline at end of file
+        with open(file='requirements.txt', mode='w') as file:
+            for package in installed_packages:
+                file.write(package + '\n')
\ No newline at end of file
